--[[
LuCI - Lua Configuration Interface

Copyright 2008 Steven Barth <steven@midlink.org>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

$Id$
]]--
require("luci.tools.webadmin")
arg[1] = arg[1] or ""

m = Map("wireless", translate("networks"), translate("a_w_networks1"))

s = m:section(NamedSection, arg[1], "wifi-device", translate("device") .. " " .. arg[1])
--s.addremove = true

en = s:option(Flag, "disabled", translate("enable"))
en.enabled = "0"
en.disabled = "1"

function en.cfgvalue(self, section)
	return Flag.cfgvalue(self, section) or "0"
end

s:option(DummyValue, "type", translate("type"))
local hwtype = m:get(arg[1], "type")

ch = s:option(Value, "channel", translate("a_w_channel"))
for i=1, 14 do
	ch:value(i, i .. " (2.4 GHz)")
end
for i=36, 64, 4 do
	ch:value(i, i .. " (5 GHz)")
end
for i=100, 140, 4 do
	ch:value(i, i .. " (5 GHz)")
end
ch:value(147, 147 .. " (5 GHz)")
ch:value(151, 151 .. " (5 GHz)")
ch:value(155, 155 .. " (5 GHz)")
ch:value(167, 167 .. " (5 GHz)")

------------------- MAC80211 Device ------------------

if hwtype == "mac80211" then

end


------------------- Madwifi Device ------------------

if hwtype == "atheros" then
	mode = s:option(ListValue, "mode", translate("mode"))
	mode:value("", translate("wifi_auto"))
	mode:value("11b", "802.11b")
	mode:value("11g", "802.11g")
	mode:value("11a", "802.11a")
	mode:value("11bg", "802.11b+g")
	mode:value("11gdt", "802.11adt")
	mode:value("11adt", "802.11adt")
	mode:value("fh", translate("wifi_fh"))
	
	s:option(Flag, "diversity", translate("wifi_diversity"))
	s:option(Value, "txantenna", translate("wifi_txantenna")).optional = true
	s:option(Value, "rxantenna", translate("wifi_rxantenna")).optional = true
	s:option(Value, "distance", translate("wifi_distance"),
		translate("wifi_distance_desc")).optional = true
	
	--s:option(Flag, "nosbeacon", translate("wifi_nosbeacon")).optional = true
end



------------------- Broadcom Device ------------------

if hwtype == "broadcom" then
	mp = s:option(ListValue, "macpolicy", translate("wifi_macpolicy"))
	mp.optional = true
	mp:value("")
	mp:value("deny", translate("wifi_whitelist"))
	mp:value("allow", translate("wifi_blacklist"))
	ml = s:option(Value, "maclist", translate("wifi_maclist"))
	ml:depends({macpolicy="allow"})
	ml:depends({macpolicy="deny"})
	
	s:option(Value, "txant", translate("wifi_txantenna")).optional = true
	s:option(Value, "rxant", translate("wifi_rxantenna")).optional = true
	
	s:option(Flag, "frameburst", translate("wifi_bursting")).optional = true
	
	s:option(Value, "distance", translate("wifi_distance")).optional = true
	--s:option(Value, "slottime", translate("wifi_slottime")).optional = true
	
	s:option(Value, "country", translate("wifi_country")).optional = true
	s:option(Value, "maxassoc", translate("wifi_maxassoc")).optional = true
end


----------------------- Interface -----------------------

s = m:section(TypedSection, "wifi-iface", translate("interfaces"))
s.addremove = true
s.anonymous = true
s:depends("device", arg[1])
s.defaults.device = arg[1]

s:option(Value, "ssid", translate("wifi_essid"))

network = s:option(Value, "network", translate("network"), translate("a_w_network1"))
network.rmempty = true
network:value("")
network.combobox_manual = translate("a_w_netmanual")
luci.tools.webadmin.cbi_add_networks(network)

function network.write(self, section, value)	
	if not m.uci:get("network", value) then 
		m:chain("network")
		m.uci:set("network", value, "interface")
		Value.write(self, section, value)
	else
		if m.uci:get("network", value) == "interface" then
			Value.write(self, section, value)
		end
	end
end


mode = s:option(ListValue, "mode", translate("mode"))
mode:value("ap", translate("a_w_ap"))
mode:value("adhoc", translate("a_w_adhoc"))
mode:value("sta", translate("a_w_client"))

bssid = s:option(Value, "bssid", translate("wifi_bssid"))


-------------------- MAC80211 Interface ----------------------

if hwtype == "mac80211" then
	mode:value("monitor", translate("a_w_monitor"))
	bssid:depends({mode="adhoc"})
	
	s:option(Value, "txpower", translate("a_w_txpwr"), "dbm").rmempty = true
	s:option(Value, "frag", translate("wifi_frag")).optional = true
	s:option(Value, "rts", translate("wifi_rts")).optional = true
end



-------------------- Madwifi Interface ----------------------

if hwtype == "atheros" then	
	mode:value("ahdemo", translate("a_w_ahdemo"))
	mode:value("monitor", translate("a_w_monitor"))
	
	bssid:depends({mode="adhoc"})
	bssid:depends({mode="ahdemo"})
	
	wds = s:option(Flag, "wds", translate("a_w_wds"))
	wds:depends({mode="ap"})
	wds:depends({mode="sta"})
	wdssep = s:option(Flag, "wdssep", translate("wifi_wdssep"))
	wdssep:depends({mode="ap", wds="1"})
	wdssep.optional = true
	
	s:option(Flag, "doth", "802.11h").optional = true
	s:option(Value, "txpower", translate("a_w_txpwr"), "dbm").rmempty = true
	hidden = s:option(Flag, "hidden", translate("wifi_hidden"))
	hidden:depends({mode="ap"})
	hidden:depends({mode="adhoc"})
	hidden:depends({mode="wds"})
	hidden.optional = true
	isolate = s:option(Flag, "isolate", translate("wifi_isolate"),
	 translate("wifi_isolate_desc"))
	isolate:depends({mode="ap"})
	isolate.optional = true
	s:option(Flag, "bgscan", translate("wifi_bgscan")).optional = true
	
	mp = s:option(ListValue, "macpolicy", translate("wifi_macpolicy"))
	mp.optional = true
	mp:value("")
	mp:value("deny", translate("wifi_whitelist"))
	mp:value("allow", translate("wifi_blacklist"))
	ml = s:option(Value, "maclist", translate("wifi_maclist"))
	ml:depends({macpolicy="allow"})
	ml:depends({macpolicy="deny"})
	
	s:option(Value, "rate", translate("wifi_rate")).optional = true
	s:option(Value, "mcast_rate", translate("wifi_mcast_rate")).optional = true
	s:option(Value, "frag", translate("wifi_frag")).optional = true
	s:option(Value, "rts", translate("wifi_rts")).optional = true
	s:option(Value, "minrate", translate("wifi_minrate")).optional = true
	s:option(Value, "maxrate", translate("wifi_maxrate")).optional = true
	s:option(Flag, "compression", translate("wifi_compression")).optional = true
	
	s:option(Flag, "bursting", translate("wifi_bursting")).optional = true
	s:option(Flag, "turbo", translate("wifi_turbo")).optional = true
	s:option(Value, "ff", translate("wifi_ff")).optional = true
	
	s:option(Flag, "wmm", translate("wifi_wmm")).optional = true
	s:option(Flag, "xr", translate("wifi_xr")).optional = true
	s:option(Flag, "ar", translate("wifi_ar")).optional = true
end


-------------------- Broadcom Interface ----------------------

if hwtype == "broadcom" then
	mode:value("wds", translate("a_w_wds"))
	mode:value("monitor", translate("a_w_monitor"))
	
	hidden = s:option(Flag, "hidden", translate("wifi_hidden"))
	hidden:depends({mode="ap"})
	hidden:depends({mode="adhoc"})
	hidden:depends({mode="wds"})
	hidden.optional = true
	
	isolate = s:option(Flag, "isolate", translate("wifi_isolate"),
	 translate("wifi_isolate_desc"))
	isolate:depends({mode="ap"})
	isolate.optional = true
	
	bssid:depends({mode="wds"})
end



------------------- WiFI-Encryption -------------------

encr = s:option(ListValue, "encryption", translate("encryption"))
encr:value("none", "No Encryption")
encr:value("wep", "WEP")
encr:value("psk", "WPA-PSK")
encr:value("psk2", "WPA2-PSK")

if hwtype == "atheros" or hwtype == "mac80211" then
	encr:value("wpa", "WPA-EAP", {mode="ap"}, {mode="sta"})
	encr:value("wpa2i", "WPA2-EAP", {mode="ap"}, {mode="sta"})
end

encr:depends("mode", "ap")
encr:depends("mode", "sta")
encr:depends("mode", "wds")

server = s:option(Value, "server", translate("a_w_radiussrv"))
server:depends({mode="ap", encryption="wpa"})
server:depends({mode="ap", encryption="wpa2i"})
server.rmempty = true

port = s:option(Value, "port", translate("a_w_radiusport"))
port:depends({mode="ap", encryption="wpa"})
port:depends({mode="ap", encryption="wpa2i"})
port.rmempty = true

key = s:option(Value, "key", translate("key"))
key:depends("encryption", "wep")
key:depends("encryption", "psk")
key:depends({mode="ap", encryption="wpa"})
key:depends("encryption", "psk2")
key:depends({mode="ap", encryption="wpa2i"})
key.rmempty = true

if hwtype == "atheros" or hwtype == "mac80211" then
	nasid = s:option(Value, "nasid", translate("a_w_nasid"))
	nasid:depends({mode="ap", encryption="wpa"})
	nasid:depends({mode="ap", encryption="wpa2i"})
	nasid.rmempty = true
	
	eaptype = s:option(ListValue, "eap_type", translate("a_w_eaptype"))
	eaptype:value("TLS")
	eaptype:value("PEAP")
	eaptype:depends({mode="sta", encryption="wpa"})
	eaptype:depends({mode="sta", encryption="wpa2i"})
	
	cacert = s:option(Value, "ca_cert", translate("a_w_cacert"))
	cacert:depends({mode="sta", encryption="wpa"})
	cacert:depends({mode="sta", encryption="wpa2i"})
	
	privkey = s:option(Value, "priv_key", translate("a_w_tlsprivkey"))
	privkey:depends({mode="sta", eap_type="TLS", encryption="wpa2i"})
	privkey:depends({mode="sta", eap_type="TLS", encryption="wpa"})
	
	privkeypwd = s:option(Value, "priv_key_pwd", translate("a_w_tlsprivkeypwd"))
	privkeypwd:depends({mode="sta", eap_type="TLS", encryption="wpa2i"})
	privkeypwd:depends({mode="sta", eap_type="TLS", encryption="wpa"})
	
	
	auth = s:option(Value, "auth", translate("a_w_peapauth"))
	auth:depends({mode="sta", eap_type="PEAP", encryption="wpa2i"})
	auth:depends({mode="sta", eap_type="PEAP", encryption="wpa"})
	
	identity = s:option(Value, "identity", translate("a_w_peapidentity"))
	identity:depends({mode="sta", eap_type="PEAP", encryption="wpa2i"})
	identity:depends({mode="sta", eap_type="PEAP", encryption="wpa"})
	
	password = s:option(Value, "password", translate("a_w_peappassword"))
	password:depends({mode="sta", eap_type="PEAP", encryption="wpa2i"})
	password:depends({mode="sta", eap_type="PEAP", encryption="wpa"})
end


return m
